/* $Modified: Fri Jun 25 23:06:31 2010 by uwer $ */
#ifndef HATHOR_H_
#define HATHOR_H_

#include "HathorFits.h"
#include "HathorVegas.h"
#include <iostream>
#include <string>
#include <sstream>
#include <cmath>
#include <cstdlib>

class Pdf {
  public:
  Pdf(){};
  virtual void GetPdf(double x, double muf, double h[13]) = 0; 
  virtual double GetAlphas(double ) = 0;
  virtual int NumberPdf(void) = 0;
  virtual void InitMember(int ) = 0;
  // virtual int InitPdf(std::string) = 0;
};

class Hathor {
 public:
  enum MASSRENORMALISATIONSCHEME {POLE_MASS=0, MS_MASS=1 };
  enum PERTURBATIVEORDER { LO=2, NLO=4, NNLO=8 };
  enum APPROXSCHEMES { LOG_ONLY=16 };
  enum SETTINGS { PDF_SCAN=32, PDF_SYM_ERR=64 };
  enum COLLIDERTYPE {PPBAR=0, PP=1};
  enum ACCURACY {LOW=1000,MEDIUM=10000,HIGH=100000};
  enum PARTONS {GLUON = 0,
		DOWN=1, UP=2, STRANGE=3, CHARM=4, BOTTOM=5, TOP=6,
		ADOWN=7, AUP=8, ASTRANGE=9, ACHARM=10, ABOTTOM=11, ATOP=12};
  
  static const double z2,z3,ln2;
  static const double ca,ca2,cf;  

  static double hcq;

  Hathor(Pdf & pdf_);

  void sethc2(double hcq_);
  void setScheme(unsigned int newscheme);
  void setColliderType( COLLIDERTYPE type);
  void setSqrtShad(double ecms);

  void setNf(int n);
  void setCqq(double tmp);
  void setCgg(double tmp);
  void setPrecision(int n);

  void PrintOptions();

  double getAlphas(double mur){return(pdf.GetAlphas(mur));}
  double getXsection(double m, double mur,double muf);
  void getResult(int pdfset, double & integral, double & err, double chi2a=0); 
  void getPdfErr(double & up, double & down);
  int getPdfNumber();

  friend class Vegas<Hathor>;

 private:
  static const double inv4pi, inv4pi2, inv4pi4;
  COLLIDERTYPE collider;
  unsigned int scheme;
  int pdfmax;
  int debug,printlevel;
  int nf;
  int calls;
  double shad;
  Pdf & pdf;
  double m,m2;
  double muf, mur, muf2, mur2, Lmr, Lmf;
  double as,as2,a;
  double b0, b1,d1,d2;
  double Cqq, Cgg;
  double fgg00,fqq00;
  double fgg10, fgg11, fqq10, fqq11, fgq10, fgq11, nlogg,nloqq,nlogq;
  double xfqq10, xfgg10, xfqq11, xfgg11;
  double fgg20, fgg21, fgg22, fqq20, fqq21, fqq22, fgq20, fgq21,fgq22;
  double nnlogg,nnloqq,nnlogq;
  double integral[100], error[100], chi2a[100];
  double as_pdf[64];
  void info(std::string s);
  void info(std::string s, double x);
  void warn(std::string s);
  void err(std::string s);
  void update();

  void evaluateNLO(double beta);
  void evaluateNNLO(double beta);
  double f(const double x[], const double wgt,double res[]);

  int getDimension(){return(2);};
  int VegasIterations(){return(10);};
  int VegasPoints(){return(calls);}
  double VegasAcc(){return(1.e-10);}

  
  static inline double fluxgg(double h1[], double h2[]){
    return(h1[GLUON] * h2[GLUON]);
  }
  
  static inline double fluxqqb(double h1[], double h2[]){
    return( // Q QB
           + h1[UP] * h2[AUP] 
	   + h1[DOWN] * h2[ADOWN]
           + h1[STRANGE] * h2[ASTRANGE] 
           + h1[CHARM] * h2[ACHARM] 
           + h1[BOTTOM] * h2[ABOTTOM] 
           // Qb Q
           + h1[AUP] * h2[UP] 
           + h1[ADOWN] * h2[DOWN]
           + h1[ASTRANGE] * h2[STRANGE] 
           + h1[ACHARM] * h2[CHARM] 
           + h1[ABOTTOM] * h2[BOTTOM] 
	    );
  }
  
  static inline double fluxqg(double h1[], double h2[]){
    return(+ h1[GLUON] 
           * ( h2[UP] + h2[DOWN] + h2[STRANGE] + h2[CHARM] + h2[BOTTOM] )
           + ( h1[UP] + h1[DOWN] + h1[STRANGE] + h1[CHARM] + h1[BOTTOM] ) 
           * h2[GLUON]
           );
  }

  static inline double fluxgqb(double h1[], double h2[]){
    return(+ h1[GLUON] 
           * ( h2[AUP] + h2[ADOWN] + h2[ASTRANGE] + h2[ACHARM] + h2[ABOTTOM] )
           + ( h1[AUP] + h1[ADOWN] + h1[ASTRANGE] + h1[ACHARM] + h1[ABOTTOM] ) 
           * h2[GLUON] );
  }

  static inline void ChargeConjugation(double h[13]){
    double tmp;
    for (int i=DOWN; i < ADOWN; i++ ){
      tmp = h[i];
      h[i] = h[i + ADOWN-DOWN];
      h[i + ADOWN-DOWN] = tmp;
    }
  }
};
#endif // HATHOR_H_
