// $Modified: Tue Ma 25 14:43:23 20: Mon May 31 19:29:55 2010 by uwer 10 by uwer $
#include "Hathor.h"
#include "HathorPdf.h"
#include <string>
#include <string.h>
#include "LHAPDF/LHAPDF.h"


using namespace std;
using namespace LHAPDF;


Lhapdf::Lhapdf(){
  PDFname  = "MSTW2008nnlo68cl";
  initPDFSet(PDFname, LHGRID);
}

Lhapdf::Lhapdf(string s){
    PDFname  = s;
    initPDFSet(PDFname, LHGRID);
}

void Lhapdf::GetPdf(double x, double muf, double h[13]){
   
  h[Hathor::GLUON] = xfx(x, muf, 0);
  h[Hathor::ABOTTOM] = xfx(x, muf, -5);
  h[Hathor::ACHARM] = xfx(x, muf, -4);
  h[Hathor::ASTRANGE] = xfx(x, muf, -3);
  h[Hathor::AUP] = xfx(x, muf, -2);
  h[Hathor::ADOWN] = xfx(x, muf, -1);
  h[Hathor::DOWN] = xfx(x, muf, 1);
  h[Hathor::UP] = xfx(x, muf, 2);
  h[Hathor::STRANGE] = xfx(x, muf, 3);
  h[Hathor::CHARM] = xfx(x, muf, 4);
  h[Hathor::BOTTOM] = xfx(x, muf, 5);

  for (int i=0; i<13; i++){
    h[i] /= x;
  }

}


double Lhapdf::GetAlphas(double mu){
  return(alphasPDF(mu) );
}

void Lhapdf::InitMember(int i){ 
  usePDFMember(i);
}

int Lhapdf::NumberPdf(void){ 
  return(numberPDF());
}


/******************************************************************/
extern "C" {
  double mstwgetonepdf_(char *,const int & iset,const double & x,
			const double &q, const int & f);
}

MSTW::MSTW() {
  std::string prefix;
  if (getenv("MSTWPATH")!=NULL) {
    prefix = std::string(getenv("MSTWPATH"));
  } else {
    prefix = std::string("./");
  }
  prefix += "mstw2008nnlo.68cl";
  strcpy(pdfname,prefix.c_str());
  imember = 0;
}

void MSTW::GetPdf(double x, double muf, double h[13]){

  h[Hathor::GLUON] = mstwgetonepdf_(pdfname,imember,x,muf,0);
  h[Hathor::ABOTTOM] = mstwgetonepdf_(pdfname,imember,x,muf,-5);
  h[Hathor::ACHARM] = mstwgetonepdf_(pdfname,imember,x,muf,-4);
  h[Hathor::ASTRANGE] = mstwgetonepdf_(pdfname,imember,x,muf,-3);
  h[Hathor::AUP] = mstwgetonepdf_(pdfname,imember,x,muf,-2);
  h[Hathor::ADOWN] = mstwgetonepdf_(pdfname,imember,x,muf,-1);
  h[Hathor::DOWN] = mstwgetonepdf_(pdfname,imember,x,muf,1);
  h[Hathor::UP] = mstwgetonepdf_(pdfname,imember,x,muf,2);
  h[Hathor::STRANGE] = mstwgetonepdf_(pdfname,imember,x,muf,3);
  h[Hathor::CHARM] = mstwgetonepdf_(pdfname,imember,x,muf,4);
  h[Hathor::BOTTOM] = mstwgetonepdf_(pdfname,imember,x,muf,5);

  for (int i=0; i<13; i++){
    h[i] /= x;
  }
}

double MSTW::GetAlphas(double mu){ return(1.);}

void MSTW::InitMember(int i){ imember=i;}

int MSTW::NumberPdf(void){return(40);}


